@@ -1,18 +1,60 @@
-# TODO: varianze funzioni + importanti
-
+#############################################################################
+#   Copyright (c) 2018 Giorgio A. Spedicato, Christophe Dutang
 #
-# Author: Giorgio Spedicato
-###############################################################################
+#   This program is free software; you can redistribute it and/or modify
+#   it under the terms of the GNU General Public License as published by
+#   the Free Software Foundation; either version 2 of the License, or
+#   (at your option) any later version.
+#
+#   This program is distributed in the hope that it will be useful,
+#   but WITHOUT ANY WARRANTY; without even the implied warranty of
+#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+#   GNU General Public License for more details.
+#
+#   You should have received a copy of the GNU General Public License
+#   along with this program; if not, write to the
+#   Free Software Foundation, Inc.,
+#   59 Temple Place, Suite 330, Boston, MA 02111-1307, USA
+#
+#############################################################################
+###
+###         actuarial functions
+###
 
-#function to obtain the endowment
-Exn <-
-  function(actuarialtable, x, n, i = actuarialtable@interest,type = "EV",power =
-             1)
+
+
+
+#' Function to evaluate the pure endowment
+#'
+#' @param actuarialtable An actuarial table object.
+#' @param x Age of the insured.
+#' @param n Length of the contract.
+#' @param i Interest rate (it overwrites the \code{actuarialtable} one)
+#' @param type A string, eithed "EV" (default value),  "ST" (stocastic realization) or "VR" if the value of the variance is needed.
+#' @param power The power of the APV. Default is 1 (mean)
+#'
+#' @return The APV of the contract
+#' @seealso \code{\link{axn}}
+#' @author Giorgio A. Spedicato
+#' @references Actuarial Mathematics (Second Edition), 1997, by Bowers, N.L., Gerber, H.U., 
+#' Hickman, J.C., Jones, D.A. and Nesbitt, C.J.
+#'
+#' @examples
+#' 
+#' #assumes SOA example life table to be load
+#' data(soaLt)
+#' soa08Act=with(soaLt, new("actuarialtable",interest=0.06, x=x,lx=Ix,name="SOA2008"))
+#' #evaluate the pure endowment for a man aged 30 for a time span of 35
+#' Exn(soa08Act, x=30, n=35) 
+#' @rdname pureendowment
+#' @export
+Exn <- function(actuarialtable, x, n, i = actuarialtable@interest, type = "EV", power = 1)
   {
     interest <- i
     out <- numeric(1)
     if (missing(actuarialtable))
       stop("Error! Need an actuarial actuarialtable") #request an actuarial actuarialtable
+    type <- testtyperesarg(type)
     prob = pxt(actuarialtable,x,n)
     discount = (1 + interest) ^ (-n)
     #defines the outputs
@@ -28,68 +70,105 @@ Exn <-
       )
     #out=discount^2*prob*(1-prob)
     return(out)
-  }
-#function to obtain the annuity
-axn <-
-  function(actuarialtable, x, n,i = actuarialtable@interest, m,k = 1, type =
-             "EV",power = 1,payment = "advance")
-  {
-    interest <- i
-    out <- numeric(1)
-    if (missing(actuarialtable))
-      stop("Error! Need an actuarial actuarialtable")
-    if (missing(x))
-      stop("Error! Need age!")
-    
-    if (x > getOmega(actuarialtable)) {
-      out = 0
-      return(out)
-    }
-    if (missing(m))
-      m = 0
-    if (missing(n))
-      n = ceiling((getOmega(actuarialtable) + 1 - x - m) * k) / k #n=getOmega(actuarialtable)-x-m Patch by Reinhold
-    if (n == 0) {
-      out = 0
-      return(out)
-    }
-    if (any(x < 0,m < 0,n < 0))
-      stop("Error! Negative parameters")
-    #computation of quantities, assuming fractional payments
-    payments = rep(1 / k,n * k)
-    probs = numeric(n * k)
-    times = m + seq(from = 0, to = (n - 1 / k),by = 1 / k)
-    if (payment == "arrears")
-      times = times + 1 / k
-    
-    for (i in 1:length(times))
-      probs[i] = pxt(actuarialtable, x,times[i])
-    discounts = (1 + interest) ^ -times #prima era asteriskato
-    #out<-sum(payments*discounts*probs)
-    if (type == "EV") {
-      out <-
-        presentValue(
-          cashFlows = payments, timeIds = times, interestRates = interest, probabilities =
-            probs,power = power
-        )
-      #out=.C("add3", x=as.double(payments), y=as.double(discounts),z=as.double(probs),n=as.integer(length(probs)),out=numeric(1))$out
-    } else if (type == "ST") {
-      out = rLifeContingencies(
-        n = 1,lifecontingency = "axn",
-        object = actuarialtable, x = x,t = n,i = interest, m = m,k = k, payment =
-          payment
-      )
-    }
-    return(out)
-  }
+}
 
 
+#function computing survival annuities
+axn <- function(actuarialtable, x, n, i = actuarialtable@interest, m,
+                    k = 1, type = "EV",power = 1, payment = "advance", ...)
+{
+  #checks
+  if (!(class(actuarialtable) %in% c("lifetable","actuarialtable")))
+    stop("Error! Only lifetable, actuarialtable classes are accepted")
+  
+  type <- testtyperesarg(type)
+  payment <- testpaymentarg(payment) # "advance"->"due"; "arrears"->"immediate"
+  
+  #class(object) %in% c("lifetable","actuarialtable") 
+  if (missing(x))
+    stop("Missing x")
+  if(length(k) > 1)
+  {
+    k <- k[1]
+    warnings("k should be of length 1, it takes first value")
+  }
+  if (missing(m))
+    m <- 0
+  if (missing(n))
+    n <- ceiling((getOmega(actuarialtable) + 1 - x - m) * k) / k 
+  if (any(x < 0, n < 0, m < 0))
+    stop("Check x, n or m")
+  
+  if(length(x) <= 0)
+    stop("x is of length zero")
+  if(length(t) <= 0)
+    stop("t is of length zero")
+  
+  ntot <- max(length(n), length(x), length(m))
+  if(length(n) != length(x) || length(n) != length(m))
+  {
+    if(length(m) != ntot)
+      warnings("m argument has been recycled to match the maximum length of x, m and n")
+    if(length(n) != ntot)
+      warnings("n argument has been recycled to match the maximum length of x, m and n")
+    if(length(x) != ntot)
+      warnings("x argument has been recycled to match the maximum length of x, m and n")
+    m <- rep(m, length.out=ntot)
+    n <- rep(n, length.out=ntot)
+    x <- rep(x, length.out=ntot)
+    if(any(is.infinite(x), is.infinite(n), is.infinite(m)))
+      stop("infinite values provided in x, n or m")
+    if(any(x < 0, n < 0, m < 0))
+      stop("(strictly) negative values provided in x, n or m")
+  }
+  
+  
+  if (type == "EV")
+  {
+    single_axn_immediate <- function(j)
+    {  
+      if(n[j] <= 0)
+        return(0)
+      #computation of quantities, assuming fractional payments
+      payments <- rep(1 / k, n[j] * k)
+      times <- m[j] + seq(from = 1/k, to = n[j], by = 1/k)
+      probs <- pxt(actuarialtable, x[j], times, ...)
+      #pxtvect(object, x, t, fractional = "linear", decrement)
+      presentValue(payments, times, i, probs, power)
+    }
+    single_axn_due <- function(j)
+    {  
+      if(n[j] <= 0)
+        return(0)
+      #computation of quantities, assuming fractional payments
+      payments <- rep(1 / k, n[j] * k)
+      times <- m[j] + seq(from = 0, to = n[j]-1/k, by = 1/k)
+      probs <- pxt(actuarialtable, x[j], times, ...)
+      presentValue(payments, times, i, probs, power)
+    }
+    if(payment == "immediate")
+      out <- sapply(1:ntot, single_axn_immediate)
+    else if(payment == "due")
+      out <- sapply(1:ntot, single_axn_due)
+    else
+      stop("wrong payment type")
+  } else if (type == "ST") {
+    
+    rng_axn <- function(j)
+      rLifeContingencies(
+        n = 1, lifecontingency = "axn", object = actuarialtable, 
+        x = x[j], t = n[j], i = i, m = m[j], k = k, payment=payment)
+    out <- sapply(1:ntot, rng_axn)
+  } else
+    stop("wrong result type")
+  out  
+}
+
 
 
 
 #shall write the Rd file
-axyn <-
-  function(tablex, tabley, x,y, n,i, m,k = 1, status = "joint", type = "EV", payment =
+axyn <- function(tablex, tabley, x,y, n,i, m,k = 1, status = "joint", type = "EV", payment =
              "advance")
   {
     .Deprecated("axyzn")
@@ -102,6 +181,10 @@ axyn <-
       stop("Error! Need age for X!")
     if (missing(y))
       stop("Error! Need age for Y!")
+    type <- testtyperesarg(type)
+    payment <- testpaymentarg(payment) # "advance"->"due"; "arrears"->"immediate"
+    status <- teststatusarg(status)
+    
     if (missing(m))
       m = 0
     if (missing(n))
@@ -119,7 +202,7 @@ axyn <-
     payments = rep(1 / k,n * k)
     probs = numeric(n * k)
     times = m + seq(from = 0, to = (n - 1 / k),by = 1 / k)
-    if (payment == "arrears")
+    if (payment == "immediate")
       times = times + 1 / k
     
     xVec = c(x,y)
@@ -148,8 +231,145 @@ axyn <-
   }
 
 
-axyzn <-
-  function(tablesList, x, n,i, m,k = 1, status = "joint", type = "EV",power =
+axyznvect <- function(tablesList, x, n, i, m, k = 1, status = "joint", type = "EV",
+                      power =1, payment = "advance", ...)
+{
+  #initial checkings
+  numTables = length(tablesList)
+  if (missing(x))
+    stop("Missing x")
+  if(!is.numeric(x))
+    stop("Error! x should be a numeric vector or a numeric matrix")
+  else if(is.vector(x))
+  {  
+    if(length(x) == 1)
+      x <- rep(x, length.out=numTables)
+    if (length(x) != numTables)
+      stop("Error! Initial ages vector length does not match with number of lives")
+  }else if(is.matrix(x))
+  {
+    if(NCOL(x) != numTables)
+      stop("Error! Age matrix colum number does not match with number of lives")
+  }else
+    stop("Error! x should be a numeric vector or a numeric matrix")
+  
+  if(is.vector(x))
+    x <- t(as.matrix(x))
+  #from here, x is a matrix where NCOL(x) == numTables
+  nbcomput <- NROW(x)
+  
+  classlist <- sapply(tablesList, class)
+  if(any(!classlist %in% c("lifetable", "actuarialtable")))
+    stop("Error! A list of lifetable objects is required")
+  
+  type <- testtyperesarg(type)
+  payment <- testpaymentarg(payment) # "advance"->"due"; "arrears"->"immediate"
+  status <- teststatusarg(status)
+  
+  #class(object) %in% c("lifetable","actuarialtable") 
+  if(length(k) > 1)
+  {
+    k <- k[1]
+    warnings("k should be of length 1, it takes first value")
+  }
+  if (missing(m))
+    m <- 0
+  if (any(x < 0, m < 0))
+    stop("Check x, n or m")
+  
+  if(length(x) <= 0)
+    stop("x is of length zero")
+  if(length(t) <= 0)
+    stop("t is of length zero")
+  
+  missingn <- missing(n)
+  if(!missingn)
+  {
+    if(length(n) <= 0)
+      stop("n is of length zero")
+    if(!is.numeric(n))
+      stop("Error! n should be a numeric vector")
+  }else
+  {
+    computn <- function(idrow)
+    {
+      getn <- function(j)
+        ceiling((getOmega(tablesList[[j]]) + 1 - x[idrow, j] - m) * k) / k 
+      myn <- max(sapply(1:numTables, getn))
+    }
+    n <- sapply(1:nbcomput, computn)
+  }
+  
+  #recycle some parameters
+  if(length(m) != nbcomput)
+  {
+    m <- rep(m, length.out = nbcomput)
+    warnings("m argument has been recycled to match the row number of x")
+  }
+  if(length(k) != nbcomput)
+  {
+    k <- rep(k, length.out = nbcomput)
+    warnings("k argument has been recycled to match the row number of x")
+  }
+  if(length(n) != nbcomput)
+  {
+    n <- rep(n, length.out = nbcomput)
+    warnings("n argument has been recycled to match the row number of x")
+  }
+  
+  
+  if (!missing(i))
+  {
+    if(!is.numeric(i))
+      stop("Error! i should be a numeric vector")
+    interest = i
+  }else 
+  {
+    interest <- sapply(1:numTables, function(j) tablesList[[j]]@interest)
+  }
+  if(length(interest) > 1)
+  {
+    interest <- mean(interest)
+    warnings("i argument is the average value of interest values")
+  }
+  
+  
+  #computation of quantities, assuming fractional payments
+  allpayments <- matrix(1 / k, nrow=nbcomput, ncol=n * k)
+  
+  computtime <- function(idrow)
+  {
+    if(payment == "immediate")
+    {  
+      times <- m[idrow] + seq(from = 1/k, to = n[idrow], by = 1/k)
+    }else if(payment == "due")
+    {  
+      times <- m[idrow] + seq(from = 0, to = n[idrow]-1/k, by = 1/k)
+    }
+    else
+      stop("wrong payment type")
+    times
+  }
+  alltime <- t(sapply(1:nbcomput, computtime))
+  
+  computprob <- function(idrow)
+  {
+    #replicate time over lifetables
+    valt <- replicate(numTables, alltime[idrow,])
+    valx <- x[idrow,]
+    pxyzt(tablesList, valx, valt, status=status, ...)
+  }
+  allprob <- t(sapply(1:nbcomput, computprob))
+  if(NROW(allprob) != nbcomput)
+    stop("wrong probabilities computed")
+  
+  res <- sapply(1:nbcomput, function(idrow)
+    presentValue(allpayments[idrow,], alltime[idrow,], interest, allprob[idrow,], power))
+  
+  res
+}
+
+axyzn <- function(tablesList, x, n,i, m,k = 1, status = "joint", type = "EV",power =
              1, payment = "advance")
   {
     out <- numeric(1)
@@ -161,6 +381,9 @@ axyzn <-
       if (!(class(tablesList[[j]]) %in% c("lifetable", "actuarialtable")))
         stop("Error! A list of lifetable objects is required")
     }
+    type <- testtyperesarg(type)
+    payment <- testpaymentarg(payment) # "advance"->"due"; "arrears"->"immediate"
+    status <- teststatusarg(status)
     
     if (k < 1)
       stop("Error! Periods in a year shall be no less than 1")
@@ -190,7 +413,7 @@ axyzn <-
     payments = rep(1 / k,n * k)
     probs = numeric(n * k)
     times = m + seq(from = 0, to = (n - 1 / k),by = 1 / k)
-    if (payment == "arrears")
+    if (payment == "immediate")
       times = times + 1 / k
     for (j in 1:length(times))
       probs[j] = pxyzt(
@@ -209,7 +432,7 @@ axyzn <-
     } else	if (type == "ST") {
       out = rLifeContingenciesXyz(
         n = 1,lifecontingency = "axyz", tablesList = tablesList, x = x,t = n,i =
-          i, m = m,k = k,status = status, payment = payment
+          i, m = m,k = k, status = status, payment = payment
       )
     }
     return(out)
@@ -235,62 +458,88 @@ axyzn <-
 #x: beginnin life age
 #m: deferring term
 #type: output requested: default expected value
-Axn <-
-  function(actuarialtable, x, n,i = actuarialtable@interest, m, k = 1, type =
-             "EV",power = 1)
+
+
+
+#function computing survival annuities
+
+Axn <- function(actuarialtable, x, n, i = actuarialtable@interest, m,
+                    k = 1, type = "EV", power = 1, ...)
+{
+  #checks
+  if (!(class(actuarialtable) %in% c("lifetable","actuarialtable")))
+    stop("Error! Only lifetable, actuarialtable classes are accepted")
+  
+  type <- testtyperesarg(type)
+  
+  #class(object) %in% c("lifetable","actuarialtable") 
+  if (missing(x))
+    stop("Missing x")
+  if(length(k) > 1)
   {
-    out <- numeric(1)
-    interest <- i
-    if (missing(actuarialtable))
-      stop("Error! Need an actuarial actuarialtable")
-    if (missing(x))
-      stop("Error! Need age!")
-    if (k < 1)
-      stop("Error! Periods in a year shall be no less than 1")
-    if (missing(m))
-      m <- 0
-    if (missing(n))
-      n <- ceiling((getOmega(actuarialtable)+1 - x - m) * k) / k  # want n to be a multiple of 1/k
-#       n = getOmega(actuarialtable) - x - m + 1 #Rosa patch
-    if (n == 0)
-      return(0)
-    if (any(x < 0,m < 0,n < 0))
-      stop("Error! Negative parameters")
-    #we have n*k possible payment times
-    payments = rep(1,n * k) #the payment is fixed
-    probs = numeric(n * k)
-    times = m + seq(from = 0, to = (n - 1 / k),by = 1 / k)
-    startAge = x
-    
-    #for(i in 1:length(times)) probs[i]=(pxt(object=actuarialtable, x=startAge,t=times[i])*qxt(object=actuarialtable, x=startAge+times[i],t=1/k))
-    for (i in 1:length(times))
-      probs[i] = .qxnt(
-        object = actuarialtable, x = startAge,n = times[i],t = 1 / k
-      )
-    
-    discounts = (1 + interest) ^ -(times + 1 / k) #prima asteriskato
-    
-    if (type == "EV") {
-      #out<-sum(payments*discounts*probs)
-      out <-
-        presentValue(
-          cashFlows = payments, timeIds = (times + 1 / k), interestRates = interest, probabilities =
-            probs,power = power
-        )
-      #out=.C("add3", x=as.double(payments), y=as.double(discounts),z=as.double(probs),n=as.integer(length(payments)),out=numeric(1))$out
-    } else if (type == "ST") {
-      out = rLifeContingencies(
-        n = 1,lifecontingency = "Axn", object = actuarialtable, x = x,t = n,i =
-          interest, m = m,k = k
-      )
-    }
-    return(out)
+    k <- k[1]
+    warnings("k should be of length 1, it takes first value")
   }
+  if (missing(m))
+    m <- 0
+  if (missing(n))
+    n <- ceiling((getOmega(actuarialtable) + 1 - x - m) * k) / k 
+  if (any(x < 0, n < 0, m < 0))
+    stop("Check x, n or m")
+  
+  if(length(x) <= 0)
+    stop("x is of length zero")
+  if(length(t) <= 0)
+    stop("t is of length zero")
+  
+  ntot <- max(length(n), length(x), length(m))
+  if(length(n) != length(x) || length(n) != length(m))
+  {
+    if(length(m) != ntot)
+      warnings("m argument has been recycled to match the maximum length of x, m and n")
+    if(length(n) != ntot)
+      warnings("n argument has been recycled to match the maximum length of x, m and n")
+    if(length(x) != ntot)
+      warnings("x argument has been recycled to match the maximum length of x, m and n")
+    m <- rep(m, length.out=ntot)
+    n <- rep(n, length.out=ntot)
+    x <- rep(x, length.out=ntot)
+    if(any(is.infinite(x), is.infinite(n), is.infinite(m)))
+      stop("infinite values provided in x, n or m")
+    if(any(x < 0, n < 0, m < 0))
+      stop("(strictly) negative values provided in x, n or m")
+  }
+  
+  
+  if (type == "EV")
+  {
+    single_Axn <- function(j)
+    {  
+      if(n[j] <= 0)
+        return(0)
+      #the payment is fixed
+      payments <- rep(1, n[j] * k)
+      times <- m[j] + seq(from = 0, to = (n[j]-1/k), by = 1/k)
+      probs <- pxt(actuarialtable, x=x[j], t=times, ...) * qxt(actuarialtable, x=x[j]+times, t=1/k, ...)
+      presentValue(payments, timeIds=(times + 1/k), interestRates=i, 
+                   probabilities=probs, power=power)
+    }
+    out <- sapply(1:ntot, single_Axn)
+    
+    
+  } else if (type == "ST") {
+    
+    rng_Axn <- function(j)
+      rLifeContingencies(
+        n = 1, lifecontingency = "Axn", object = actuarialtable, 
+        x = x[j], t = n[j], i = i, m = m[j], k = k)
+    out <- sapply(1:ntot, rng_Axn)
+  } else
+    stop("wrong result type")
+  out  
+}
 
-#Axn(soa08Act,65,k=12, n=12) # 0.2483798
-
-Axyn <-
-  function(tablex, x,tabley, y, n,i, m, k = 1, status = "joint", type = "EV")
+Axyn <- function(tablex, x,tabley, y, n,i, m, k = 1, status = "joint", type = "EV")
   {
     .Deprecated("Axyzn")
     out <- numeric(1)
@@ -298,6 +547,9 @@ Axyn <-
       stop("Error! Need tables")
     if (any(missing(x),missing(y)))
       stop("Error! Need ages!")
+    type <- testtyperesarg(type)
+    status <- teststatusarg(status)
+    
     if (k < 1)
       stop("Error! Periods in a year shall be no less than 1")
     if (missing(m))
@@ -350,8 +602,7 @@ Axyn <-
   }
 
 
-Axyzn <-
-  function(tablesList, x, n,i, m, k = 1, status = "joint", type = "EV",power =
+Axyzn <- function(tablesList, x, n,i, m, k = 1, status = "joint", type = "EV",power =
              1)
   {
     out = numeric(1)
@@ -363,6 +614,8 @@ Axyzn <-
       if (!(class(tablesList[[j]]) %in% c("lifetable", "actuarialtable")))
         stop("Error! A list of lifetable objects is required")
     }
+    type <- testtyperesarg(type)
+    status <- teststatusarg(status)
     
     if (k < 1)
       stop("Error! Periods in a year shall be no less than 1")
@@ -446,8 +699,7 @@ Axyzn <-
 # m=0
 # k=2
 
-IAxn <-
-  function(actuarialtable, x, n,i = actuarialtable@interest, m = 0, k = 1, type =
+IAxn <- function(actuarialtable, x, n,i = actuarialtable@interest, m = 0, k = 1, type =
              "EV",power = 1)
   {
     out <- numeric(1)
@@ -458,6 +710,7 @@ IAxn <-
       m = 0
     if (missing(x))
       stop("Error! Need age!")
+    type <- testtyperesarg(type)
     
     if (missing(n))
       n = getOmega(actuarialtable) - x - m #n=getOmega(actuarialtable)-x-m-1
@@ -515,8 +768,7 @@ IAxn <-
 # IAxn(soa08Act, x=50,n=10,k=2,type="EV")
 # IAxn(soa08Act, x=50,n=10,k=2,type="ST")
 
-DAxn <-
-  function(actuarialtable, x, n,i = actuarialtable@interest, m = 0, k = 1, type =
+DAxn <- function(actuarialtable, x, n,i = actuarialtable@interest, m = 0, k = 1, type =
              "EV",power = 1)
   {
     out <- numeric(1)
@@ -525,6 +777,7 @@ DAxn <-
       stop("Error! Need an actuarial actuarialtable")
     if (missing(x))
       stop("Error! Need age!")
+    type <- testtyperesarg(type)
     if (missing(m))
       m = 0
     if (missing(n))
@@ -576,14 +829,14 @@ DAxn <-
 
 #n-year increasing
 #recursive function
-Iaxn <-
-  function(actuarialtable, x, n,i = actuarialtable@interest, m = 0, type =
-             "EV",power = 1)
+Iaxn <- function(actuarialtable, x, n,i = actuarialtable@interest, m = 0, type =
+             "EV", power = 1)
   {
     out <- numeric(1)
     interest <- i
     if (missing(actuarialtable))
       stop("Error! Need an actuarial actuarialtable")
+    type <- testtyperesarg(type)
     if (missing(m))
       m = 0
     if (missing(x))
@@ -611,8 +864,7 @@ Iaxn <-
 
 
 #pure endowment function
-AExn <-
-  function(actuarialtable, x, n,i = actuarialtable@interest, k = 1, type =
+AExn <- function(actuarialtable, x, n,i = actuarialtable@interest, k = 1, type =
              "EV",power = 1)
   {
     out <- numeric(1)
@@ -621,6 +873,7 @@ AExn <-
       stop("Error! Need an actuarial actuarialtable")
     if (missing(x))
       stop("Error! Need age!")
+    type <- testtyperesarg(type)
     if (k < 1)
       stop("Error! Periods in a year shall be no less than 1")
     if (missing(n))
